-- IS61LV25616_model.vhd
--
-- IS61LV25616̃fB^C~O֌W͑Slɓ삾f
-- 256KBƌ̂ŁA1/4ɂ

library IEEE;
use IEEE.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

library modelsim_lib;
use modelsim_lib.util.all;

entity IS61LV25616 is
	port(
		mem_data : inout std_logic_vector(15 downto 0);	-- SRAMmemory data
		n_mem_we : in std_logic; -- SRAMmemory write enable
		n_mem_rd : in std_logic; -- SRAMmemory read enable
		n_mem_cs : in std_logic; -- SRAMchip select 0
		n_mem_lowerB : in std_logic; -- SRAMmemory lower byte enable
		n_mem_upperB : in std_logic; -- SRAMmemory upper byte enable
		mem_addr : in std_logic_vector(17 downto 0) -- SRAMmemory address
	);
end IS61LV25616;

architecture RTL of IS61LV25616 is
type SRAM256K is array (0 to 65535) of bit_vector(7 downto 0);
signal sram_upper : SRAM256K;
signal sram_lower : SRAM256K;
signal mem_addr_int : integer range 0 to 65535;
signal mem_data_node : std_logic_vector(15 downto 0);
signal mem_data_node2 : std_logic_vector(15 downto 0);
signal master_sync : std_logic;

begin
	mem_addr_int <= CONV_INTEGER(mem_addr(15 downto 0));
	
	-- Write Upper
	process(n_mem_we, n_mem_cs, n_mem_upperB, mem_data, mem_addr_int) begin
		if n_mem_we='0' and n_mem_cs='0' and n_mem_upperB='0' then
			sram_upper(mem_addr_int) <= To_bitvector(mem_data(15 downto 8));
		end if;
	end process;
	
	-- Write Lower
	process(n_mem_we, n_mem_cs, n_mem_lowerB, mem_data, mem_addr_int) begin
		if n_mem_we='0' and n_mem_cs='0' and n_mem_lowerB='0' then
			sram_lower(mem_addr_int) <= To_bitvector(mem_data(7 downto 0));
		end if;
	end process;
	
	-- -- Read Upper
	-- process(n_mem_cs, n_mem_rd, n_mem_upperB, mem_addr_int) begin
		-- if n_mem_cs='0' and n_mem_rd='0' and n_mem_upperB='0' then
			-- mem_data_node(15 downto 8) <= To_stdlogicvector(sram_upper(mem_addr_int));
		-- else
			-- mem_data_node(15 downto 8) <= (others => 'Z');
		-- end if;
	-- end process;
	
	-- -- Read Lower
	-- process(n_mem_cs, n_mem_rd, n_mem_lowerB, mem_addr_int) begin
		-- if n_mem_cs='0' and n_mem_rd='0' and n_mem_lowerB='0' then
			-- mem_data_node(7 downto 0) <= To_stdlogicvector(sram_lower(mem_addr_int));
		-- else
			-- mem_data_node(7 downto 0) <= (others => 'Z');
		-- end if;
	-- end process;
	
	-- init_signal_spy IS61LV25616_inst̉master_syncĂ
	process begin
		init_signal_spy("../camdispcntrl_sram_inst/master_sync", "master_sync", 1, -1);
		wait;
	end process;
		
	-- f[^̗₷悤ɏʂ̃f[^55+1, ʂ̃f[^AA+1āAo
	process(master_sync, n_mem_we) begin -- Read Upper
		if master_sync='1' then
			mem_data_node2(15 downto 8) <= x"55";
		elsif n_mem_we'event and n_mem_we='1' then
			if n_mem_cs='0' and n_mem_upperB='0' then
				mem_data_node2(15 downto 8) <= mem_data_node2(15 downto 8) + 1;
			end if;
		end if;
	end process;
	mem_data_node(15 downto 8) <= mem_data_node2(15 downto 8) when n_mem_cs='0' and n_mem_rd='0' and n_mem_upperB='0' else (others => 'Z');
		
	process(master_sync, n_mem_we) begin -- Read Lower
		if master_sync='1' then
			mem_data_node2(7 downto 0) <= x"AA";
		elsif n_mem_we'event and n_mem_we='1' then
			if n_mem_cs='0' and n_mem_lowerB='0' then
				mem_data_node2(7 downto 0) <= mem_data_node2(7 downto 0) + 1;
			end if;
		end if;
	end process;
	mem_data_node(7 downto 0) <= mem_data_node2(7 downto 0) when n_mem_cs='0' and n_mem_rd='0' and n_mem_lowerB='0' else (others => 'Z');
		
	mem_data <= mem_data_node after 1 ns;
end RTL;

	