-- OV7640_Model.vhd
-- ݂OV7640̎dl@\݂̂f
-- Yf[^0+1ĂBUf[^FF2Zi܂肸ƊjAVf[^FE2Zi܂肸Ƌj
-- clkɑ΂pclk5nsxx

library IEEE;
use IEEE.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity OV7640_Model is
	port(
		clk : in std_logic; -- NbN
		reset : in std_logic; -- Zbg
		vsync : out std_logic;
		href : out std_logic;
		pclk : out std_logic;
		ydata : out std_logic_vector(7 downto 0)
		-- sio_c : in std_logic;
		-- sio_d : inout std_logic
	);
end OV7640_Model;

architecture BEHAVIOR of OV7640_Model is
constant H_ACTIVE_VIDEO : integer := 640*2;
constant H_BLANK_SPACE : integer := 124*2;
constant H_SUM : integer := H_ACTIVE_VIDEO + H_BLANK_SPACE;

constant V_ACTIVE_VIDEO : integer := 480;
constant V_FRONT_PORCH : integer := 31;
constant V_SYNC_PULSE : integer := 3;
constant V_BACK_PORCH : integer := 11;
constant V_SUM : integer := V_ACTIVE_VIDEO + V_FRONT_PORCH + V_SYNC_PULSE + V_BACK_PORCH;

signal h_count : unsigned(10 downto 0) := (others => '0'); -- JE^
signal v_count : unsigned(9 downto 0) := unsigned(CONV_STD_LOGIC_VECTOR((V_ACTIVE_VIDEO + V_FRONT_PORCH)-1,10)); -- JE^
signal u_data : unsigned(7 downto 0) := (others => '1');
signal v_data : unsigned(7 downto 0) := X"FE";
signal y_data : unsigned(7 downto 0) := (others => '0');
signal vsync_node : std_logic;
signal ydata_node : std_logic_vector(7 downto 0);
signal pclock : std_logic;
signal href_node : std_logic;
begin
	pclock <= transport clk after 5 ns;
	pclk <= pclock;
	
	-- ^C~O
	process(reset, pclock) begin
		if reset='1' then
			h_count <= (others => '0');
		elsif pclock'event and pclock='1' then
			if h_count < H_SUM-1 then
				h_count <= h_count + 1;
			else -- CMOSJUYȂǂ2NbN1f̂
				h_count <= (others => '0');
			end if;
		end if;
	end process;
	href_node <= '1' when h_count<H_ACTIVE_VIDEO and v_count<V_ACTIVE_VIDEO else '0';
	href <= href_node after 1 ns;
	
	-- ^C~O
	process(reset, pclock) begin
		if reset='1' then
			v_count <= unsigned(CONV_STD_LOGIC_VECTOR((V_ACTIVE_VIDEO + V_FRONT_PORCH)-1,10));
		elsif pclock'event and pclock='1' then
			if h_count=H_SUM-1 then
				if v_count < V_SUM-1 then
					v_count <= v_count + 1;
				else
					v_count <= (others => '0');
				end if;
			end if;
		end if;
	end process;
	
	-- vsync ̏o̓^C~O
	process(v_count) begin
		if v_count>=(V_ACTIVE_VIDEO + V_FRONT_PORCH)-1 then
			if v_count<V_ACTIVE_VIDEO + V_FRONT_PORCH + V_SYNC_PULSE then
				vsync_node <= '1';
			else
				vsync_node <= '0';
			end if;
		else
			vsync_node <= '0';
		end if;
	end process;
	vsync <= vsync_node after 1 ns;
	
	-- ydata ̏o
	process(reset, pclock) begin -- Yf[^
		if reset='1' then
			y_data <= (others => '0');
		elsif pclock'event and pclock='1' then
			if h_count<H_ACTIVE_VIDEO and h_count(0)='1' then -- ̕\Ԃ2NbN1JEgAbv
				y_data <= y_data + 1;
			elsif h_count=0 and v_count=0 then -- Pt[`̍ŏɃZbg
				y_data <= (others => '0');
			end if;
		end if;
	end process;
	
	process(reset, pclock) begin -- Uf[^
		if reset='1' then
			u_data <= (others => '1');
		elsif pclock'event and pclock='1' then
			if h_count<H_ACTIVE_VIDEO and h_count(1 downto 0)=0 then -- ̕\Ԃ4NbN1-2
				u_data <= u_data - 2;
			elsif h_count=0 and v_count=0 then -- Pt[`̍ŏɃZbg
				u_data <= (others => '1');
			end if;
		end if;
	end process;
	
	process(reset, pclock) begin -- Vf[^
		if reset='1' then
			v_data <= X"FE";
		elsif pclock'event and pclock='1' then
			if h_count<H_ACTIVE_VIDEO and h_count(1 downto 0)=2 then -- ̕\Ԃ4NbN1-2
				v_data <= v_data - 2;
			elsif h_count=0 and v_count=0 then -- Pt[`̍ŏɃZbg
				v_data <= X"FE";
			end if;
		end if;
	end process;

	process(h_count, y_data, u_data, v_data) begin
		if h_count<H_ACTIVE_VIDEO then -- \
			case h_count(1 downto 0) is
				when "00" =>
					ydata_node <= std_logic_vector(u_data);
				when "01" =>
					ydata_node <= std_logic_vector(y_data);
				when "10" =>
					ydata_node <= std_logic_vector(v_data);
				when others => -- "11"
					ydata_node <= std_logic_vector(y_data);
			end case;
		else
			ydata_node <= (others => '0');
		end if;
	end process;
	ydata <= ydata_node after 1 ns;
end BEHAVIOR;

	