-- DCM Module (24MHz)

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
-- pragma translate_off
library unisim;
use unisim.vcomponents.all;
-- pragma translate_on

entity DCM_module_24MHz is
	port(
		clk48MHz_in : in std_logic;
		clk48_out : out std_logic;
		cam_clk_out : out std_logic;
		pclk_in : in std_logic;
		mclk_out : out std_logic;
		cam_clk_locked : out std_logic;
		mclk_locked : out std_logic
	);
end DCM_module_24MHz;

architecture RTL of DCM_module_24MHz is
component DCM
  generic (
    CLKDV_DIVIDE : real := 2.0;
    CLKFX_DIVIDE : integer := 1;
    CLKFX_MULTIPLY : integer := 4;
    CLKIN_DIVIDE_BY_2 : boolean := false;
    CLKIN_PERIOD : real := 10.0;                         --non-simulatable
    CLKOUT_PHASE_SHIFT : string := "NONE";
    CLK_FEEDBACK : string := "1X";
    DESKEW_ADJUST : string := "SYSTEM_SYNCHRONOUS";     --non-simulatable
    DFS_FREQUENCY_MODE : string := "LOW";
    DLL_FREQUENCY_MODE : string := "LOW";
    DSS_MODE : string := "NONE";                        --non-simulatable
    DUTY_CYCLE_CORRECTION : boolean := true;
    FACTORY_JF : bit_vector := X"C080";                 --non-simulatable
    PHASE_SHIFT : integer := 0;
    STARTUP_WAIT : boolean := false                     --non-simulatable
    );

  port (
    CLK0 : out std_ulogic := '0';
    CLK180 : out std_ulogic := '0';
    CLK270 : out std_ulogic := '0';
    CLK2X : out std_ulogic := '0';
    CLK2X180 : out std_ulogic := '0';
    CLK90 : out std_ulogic := '0';
    CLKDV : out std_ulogic := '0';
    CLKFX : out std_ulogic := '0';
    CLKFX180 : out std_ulogic := '0';
    LOCKED : out std_ulogic := '0';
    PSDONE : out std_ulogic := '0';
    STATUS : out std_logic_vector(7 downto 0) := "00000000";
    
    CLKFB : in std_ulogic := '0';
    CLKIN : in std_ulogic := '0';
    DSSEN : in std_ulogic := '0';
    PSCLK : in std_ulogic := '0';
    PSEN : in std_ulogic := '0';
    PSINCDEC : in std_ulogic := '0';
    RST : in std_ulogic := '0'
    );
end component;

component bufg
  port(
	i : in std_ulogic;
	o : out std_ulogic
	);
end component;

component ibufg
  port(
	i : in std_ulogic;
	o : out std_ulogic
	);
end component;

signal IBUFG_out : std_logic;
signal osc_48MHz, osc_48MHz_bufg : std_logic;
signal cam_clk : std_logic;
signal pclk_clk0, pclk_bufg : std_logic;
signal clk48_node : std_logic;

begin
	IBUFG_48MHz : IBUFG port map(
		i => clk48MHz_in,
		o => IBUFG_out
	);
	
	DCM_cam_clk : DCM generic map( -- 48MHz
		CLKIN_PERIOD => 20.833,
		CLKDV_DIVIDE => 2.0,
		DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS"
--		FACTORY_JF => X"FFFF"
	)port map(
		clkin => IBUFG_out,
		clkfb => osc_48MHz_bufg,
		dssen => '0',
		psincdec => '0',
		psen => '0',
		psclk => IBUFG_out,
		rst => '0',
		clk0 => osc_48MHz,
		clkdv => cam_clk,
		locked => cam_clk_locked
	);
	
	bufg48 : bufg port map(
		i => osc_48MHz,
		o => osc_48MHz_bufg
	);
	
	bufg_cam_clk : bufg port map(
		i => cam_clk,
		o => cam_clk_out
	);
	
	DCM_pclk : DCM generic map( -- 24MHz
		CLKIN_PERIOD => 41.667,
		CLKDV_DIVIDE => 2.0,
		DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS"
--		FACTORY_JF => X"FFFF"
	)port map(
		clkin => pclk_in,
		clkfb => pclk_bufg,
		dssen => '0',
		psincdec => '0',
		psen => '0',
		psclk => pclk_in,
		rst => '0',
		clk0 => pclk_clk0,
		clk2x => clk48_node,
		clkdv => open,
		locked => mclk_locked
	);
	
	bufg_pclk : bufg port map(
		i => pclk_clk0,
		o => pclk_bufg
	);
	mclk_out <= pclk_bufg;
	
	bufg_48MHz_out : bufg port map(
		i => clk48_node,
		o => clk48_out
	);
	
end RTL;
